/*
 * Copyright (c) 2012 Qualcomm Atheros, Inc.
 * All Rights Reserved.
 * Qualcomm Atheros Confidential and Proprietary.
 * Notifications and licenses are retained for attribution purposes only
 */

/*
 * Copyright (c) 2004, Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 *
 * $Id: //depot/sw/switch/app/elite/trunk/elite/os/contiki/contiki-2.5/core/net/tcpip.c#5 $
 */
/**
 * \file
 *         Code for tunnelling uIP packets over the Rime mesh routing module
 *
 * \author  Adam Dunkels <adam@sics.se>\author
 * \author  Mathilde Durvy <mdurvy@cisco.com> (IPv6 related code)
 * \author  Julien Abeille <jabeille@cisco.com> (IPv6 related code)
 */
#include "contiki-net.h"

#include "net/uip-split.h"

#include "net/uip-packetqueue.h"

#include <string.h>

#if UIP6_CONF_IPV6
#include "net/uip-nd6.h"
#include "net/uip-ds6.h"

static int uip6_ipv6_status = UIP6_DISABLE;
static int uip6_slaac_status = UIP6_DISABLE;

#endif

#define DEBUG 0
#if DEBUG
#include <stdio.h>
#define PRINTF(...) printf(__VA_ARGS__)
#define PRINT6ADDR(addr) PRINTF(" %02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x ", ((u8_t *)addr)[0], ((u8_t *)addr)[1], ((u8_t *)addr)[2], ((u8_t *)addr)[3], ((u8_t *)addr)[4], ((u8_t *)addr)[5], ((u8_t *)addr)[6], ((u8_t *)addr)[7], ((u8_t *)addr)[8], ((u8_t *)addr)[9], ((u8_t *)addr)[10], ((u8_t *)addr)[11], ((u8_t *)addr)[12], ((u8_t *)addr)[13], ((u8_t *)addr)[14], ((u8_t *)addr)[15])
#define PRINTLLADDR(lladdr) PRINTF(" %02x:%02x:%02x:%02x:%02x:%02x ",lladdr->addr[0], lladdr->addr[1], lladdr->addr[2], lladdr->addr[3],lladdr->addr[4], lladdr->addr[5])
#else
#define PRINTF(...)
#define PRINT6ADDR(addr)
#endif

#if UIP_LOGGING
#include <stdio.h>
void uip_log(char *msg);
#define UIP_LOG(m) uip_log(m)
#else
#define UIP_LOG(m)
#endif

#define UIP_ICMP_BUF ((struct uip_icmp_hdr *)&uip_buf[UIP_LLIPH_LEN + uip_ext_len])
#define UIP_IP_BUF ((struct uip_ip_hdr *)&uip_buf[UIP_LLH_LEN])
#define UIP_TCP_BUF ((struct uip_tcpip_hdr *)&uip_buf[UIP_LLH_LEN])
#if UIP6_CONF_IPV6
#define UIP6_IP_BUF ((struct uip6_ip_hdr *)&uip_buf[UIP_LLH_LEN])
#define UIP6_TCP_BUF ((struct uip6_tcpip_hdr *)&uip_buf[UIP_LLH_LEN])
#endif

#ifdef UIP_FALLBACK_INTERFACE
extern struct uip_fallback_interface UIP_FALLBACK_INTERFACE;
#endif
#if UIP_CONF_IPV6_RPL
void rpl_init(void);
#endif
process_event_t tcpip_event;
#if UIP_CONF_ICMP6
process_event_t tcpip_icmp6_event;
#endif /* UIP_CONF_ICMP6 */

/*static struct tcpip_event_args ev_args;*/

/*periodic check of active connections*/
static struct etimer periodic;

#if UIP6_CONF_IPV6 && UIP_CONF_IPV6_REASSEMBLY
/*timer for reassembly*/
extern struct etimer uip_reass_timer;
#endif

#if UIP_TCP
/**
 * \internal Structure for holding a TCP port and a process ID.
 */
struct listenport {
  u16_t port;
  struct process *p;
};

static struct internal_state {
  struct listenport listenports[UIP_LISTENPORTS];
  struct process *p;
} s;
#endif

enum {
  TCP_POLL,
  UDP_POLL,
  PACKET_INPUT,
  EV_ENABLE_IPV6,
  START_SLAAC,
};

/* Called on IP packet output. */
#if UIP6_CONF_IPV6
extern void set_ath_header_to_zero(void);
extern void gmac_send(void);
extern void wpcap_send(void);
#define BUF ((struct uip_eth_hdr *)&uip_buf[0])
#define IP6BUF ((struct uip6_tcpip_hdr *)&uip_buf[UIP_LLH_LEN])

u8_t l2_output6(uip_lladdr_t *lladdr)
{
    set_ath_header_to_zero();

    if(lladdr == NULL)
    {
        /* the dest must be multicast*/
        (&BUF->dest)->addr[0] = 0x33;
        (&BUF->dest)->addr[1] = 0x33;
        (&BUF->dest)->addr[2] = IP6BUF->destipaddr.u8[12];
        (&BUF->dest)->addr[3] = IP6BUF->destipaddr.u8[13];
        (&BUF->dest)->addr[4] = IP6BUF->destipaddr.u8[14];
        (&BUF->dest)->addr[5] = IP6BUF->destipaddr.u8[15];
    }
    else
    {
        //when fallback used this gets ptr to lladdr of all zeroes on forwarded packets, turn them into multicast(?)
        if ((lladdr->addr[0]==0)&&(lladdr->addr[1]==0)&&(lladdr->addr[2]==0)&&(lladdr->addr[3]==0)&&(lladdr->addr[4]==0)&&(lladdr->addr[5]==0))
        {
            (&BUF->dest)->addr[0] = 0x33;
            (&BUF->dest)->addr[1] = 0x33;
            (&BUF->dest)->addr[2] = IP6BUF->destipaddr.u8[12];
            (&BUF->dest)->addr[3] = IP6BUF->destipaddr.u8[13];
            (&BUF->dest)->addr[4] = IP6BUF->destipaddr.u8[14];
            (&BUF->dest)->addr[5] = IP6BUF->destipaddr.u8[15];
        }
        else
        {
            memcpy(&BUF->dest, lladdr, UIP_LLADDR_LEN);
        }
    }
    memcpy(&BUF->src, &uip_lladdr, UIP_LLADDR_LEN);
    BUF->type = UIP_HTONS(UIP_ETHTYPE_IPV6);
    uip_len += sizeof(struct uip_eth_hdr);
#ifndef WIN32
    gmac_send();
#else
    wpcap_send();
#endif

  return 0;
}

static u8_t (* outputfunc6)(uip_lladdr_t *a) = l2_output6;

u8_t
tcpip_output6(uip_lladdr_t *a)
{
  int ret;
  if(outputfunc6 != NULL) {
    ret = outputfunc6(a);
    return ret;
  }
  UIP_LOG("tcpip_output6: Use tcpip_set_outputfunc() to set an output function");
  return 0;
}

void
tcpip_set_outputfunc6(u8_t (*f)(uip_lladdr_t *))
{
  outputfunc6 = f;
}
#endif

static u8_t (* outputfunc)(void);
u8_t
tcpip_output(void)
{
  if(outputfunc != NULL) {
    return outputfunc();
  }
  UIP_LOG("tcpip_output: Use tcpip_set_outputfunc() to set an output function");
  return 0;
}

void
tcpip_set_outputfunc(u8_t (*f)(void))
{
  outputfunc = f;
}

#if UIP_CONF_IP_FORWARD
unsigned char tcpip_is_forwarding; /* Forwarding right now? */
#endif /* UIP_CONF_IP_FORWARD */

PROCESS(tcpip_process, "TCP/IP stack");

/*---------------------------------------------------------------------------*/
static void
start_periodic_tcp_timer(void)
{
  if(etimer_expired(&periodic)) {
    etimer_restart(&periodic);
  }
}
/*---------------------------------------------------------------------------*/
static void
check_for_tcp_syn(int ip_v)
{
  /* This is a hack that is needed to start the periodic TCP timer if
     an incoming packet contains a SYN: since uIP does not inform the
     application if a SYN arrives, we have no other way of starting
     this timer.  This function is called for every incoming IP packet
     to check for such SYNs. */
#define TCP_SYN 0x02
#if UIP6_CONF_IPV6
    if(ip_v==IP_VER_6)
    {
        if(UIP6_IP_BUF->proto == UIP_PROTO_TCP &&(UIP6_TCP_BUF->flags & TCP_SYN) == TCP_SYN)
        {
            start_periodic_tcp_timer();
        }
    }
    else
#endif
    {
        if(UIP_IP_BUF->proto == UIP_PROTO_TCP &&(UIP_TCP_BUF->flags & TCP_SYN) == TCP_SYN)
        {
            start_periodic_tcp_timer();
        }
    }
}
/*---------------------------------------------------------------------------*/
static void
packet_input(process_data_t data)
{
  int ip_v = *(int*)data;

#if UIP6_CONF_IPV6
  if(ip_v==IP_VER_6 && uip6_ipv6_status != UIP6_ENABLE)
  {/*recieve ipv6 packet, but ipv6 is disable*/
    return;
  }
#endif
#if UIP_CONF_IP_FORWARD
  if(uip_len > 0) {
    tcpip_is_forwarding = 1;
    if(uip_fw_forward() == UIP_FW_LOCAL) {
      tcpip_is_forwarding = 0;
      check_for_tcp_syn(ip_v);
      uip_input();
      if(uip_len > 0) {
        uip_split_output();
#if UIP6_CONF_IPV6
        if(ip_v==IP_VER_6)
        {
    #if UIP_CONF_TCP_SPLIT
            uip_split_output6();
    #else /* UIP_CONF_TCP_SPLIT */
            tcpip_ipv6_output();
    #endif /* UIP_CONF_TCP_SPLIT */
        }
        else
#endif
        {
        	PRINTF("tcpip packet_input forward output len %d\n", uip_len);
    #if UIP_CONF_TCP_SPLIT
           uip_split_output();
    #else /* UIP_CONF_TCP_SPLIT */
           tcpip_output();
    #endif /* UIP_CONF_TCP_SPLIT */
        }
      }
    }
    tcpip_is_forwarding = 0;
  }
#else /* UIP_CONF_IP_FORWARD */
  if(uip_len > 0) {
    check_for_tcp_syn(ip_v);
#if UIP6_CONF_IPV6
	if(ip_v==IP_VER_6)
    {
        uip6_process(UIP_DATA);
    }
    else
#endif
    {
        uip_process(UIP_DATA);
    }
    //uip_input();
    if(uip_len > 0) {
#if UIP6_CONF_IPV6
        if(ip_v==IP_VER_6)
        {
    #if UIP_CONF_TCP_SPLIT
            uip_split_output6();
    #else /* UIP_CONF_TCP_SPLIT */
            tcpip_ipv6_output();
    #endif /* UIP_CONF_TCP_SPLIT */
        }
        else
#endif
        {
        	PRINTF("tcpip packet_input forward output len %d\n", uip_len);
    #if UIP_CONF_TCP_SPLIT
            uip_split_output();
    #else /* UIP_CONF_TCP_SPLIT */
            tcpip_output();
    #endif /* UIP_CONF_TCP_SPLIT */
        }
    }
  }
#endif /* UIP_CONF_IP_FORWARD */
}
/*---------------------------------------------------------------------------*/
#if UIP_TCP
#if UIP_ACTIVE_OPEN
struct uip_conn *
tcp_connect(uip_ipaddr_comm_t *ripaddr, u16_t port, void *appstate)
{
  struct uip_conn *c;

  c = uip_connect(ripaddr, port);
  if(c == NULL) {
    return NULL;
  }

  c->appstate.p = PROCESS_CURRENT();
  c->appstate.state = appstate;

  tcpip_poll_tcp(c);

  return c;
}
#endif /* UIP_ACTIVE_OPEN */
/*---------------------------------------------------------------------------*/
void
tcp_unlisten(u16_t port)
{
  static unsigned char i;
  struct listenport *l;

  l = s.listenports;
  for(i = 0; i < UIP_LISTENPORTS; ++i) {
    if(l->port == port &&
       l->p == PROCESS_CURRENT()) {
      l->port = 0;
      uip_unlisten(port);
      break;
    }
    ++l;
  }
}
/*---------------------------------------------------------------------------*/
void
tcp_listen(u16_t port)
{
  static unsigned char i;
  struct listenport *l;

  l = s.listenports;
  for(i = 0; i < UIP_LISTENPORTS; ++i) {
    if(l->port == 0) {
      l->port = port;
      l->p = PROCESS_CURRENT();
      uip_listen(port);
      break;
    }
    ++l;
  }
}
/*---------------------------------------------------------------------------*/
void
tcp_attach(struct uip_conn *conn,
	   void *appstate)
{
  register uip_tcp_appstate_t *s;

  s = &conn->appstate;
  s->p = PROCESS_CURRENT();
  s->state = appstate;
}

#endif /* UIP_TCP */
/*---------------------------------------------------------------------------*/
#if UIP_UDP
void
udp_attach(struct uip_udp_conn *conn,
	   void *appstate)
{
  register uip_udp_appstate_t *s;

  s = &conn->appstate;
  s->p = PROCESS_CURRENT();
  s->state = appstate;
}
/*---------------------------------------------------------------------------*/
struct uip_udp_conn *
udp_new(const uip_ipaddr_comm_t *ripaddr, u16_t port, void *appstate)
{
  struct uip_udp_conn *c;
  uip_udp_appstate_t *s;

  c = uip_udp_new(ripaddr, port);
  if(c == NULL) {
    return NULL;
  }

  s = &c->appstate;
  s->p = PROCESS_CURRENT();
  s->state = appstate;

  return c;
}
/*---------------------------------------------------------------------------*/
struct uip_udp_conn *
udp_broadcast_new(u16_t port, void *appstate)
{
  uip_ipaddr_comm_t addr;
  struct uip_udp_conn *conn;

  uip_ipaddr(&addr, 255,255,255,255);
  conn = udp_new(&addr, port, appstate);
  if(conn != NULL) {
    udp_bind(conn, port);
  }
  return conn;
}
#if UIP6_CONF_IPV6
struct uip_udp_conn *
udp_broadcast_new6(u16_t port, void *appstate)
{
  uip_ipaddr_comm_t addr;
  struct uip_udp_conn *conn;

  uip_create_linklocal_allnodes_mcast(&addr);
  conn = udp_new(&addr, port, appstate);
  if(conn != NULL) {
    udp_bind(conn, port);
  }
  return conn;
}
#endif/*UIP6_CONF_IPV6*/
#endif /* UIP_UDP */
/*---------------------------------------------------------------------------*/
#if UIP_CONF_ICMP6
u8_t
icmp6_new(void *appstate) {
  if(uip_icmp6_conns.appstate.p == PROCESS_NONE) {
    uip_icmp6_conns.appstate.p = PROCESS_CURRENT();
    uip_icmp6_conns.appstate.state = appstate;
    return 0;
  }
  return 1;
}

void
tcpip_icmp6_call(u8_t type)
{
  if(uip_icmp6_conns.appstate.p != PROCESS_NONE) {
    /* XXX: This is a hack that needs to be updated. Passing a pointer (&type)
       like this only works with process_post_synch. */
    process_post_synch(uip_icmp6_conns.appstate.p, tcpip_icmp6_event, &type);
  }
  return;
}
#endif /* UIP_CONF_ICMP6 */
/*---------------------------------------------------------------------------*/
static void
eventhandler(process_event_t ev, process_data_t data)
{
#if UIP_TCP
  static unsigned char i;
  register struct listenport *l;
#endif /*UIP_TCP*/
  struct process *p;
  switch(ev) {
    case PROCESS_EVENT_EXITED:
      /* This is the event we get if a process has exited. We go through
         the TCP/IP tables to see if this process had any open
         connections or listening TCP ports. If so, we'll close those
         connections. */

      p = (struct process *)data;
#if UIP_TCP
      l = s.listenports;
      for(i = 0; i < UIP_LISTENPORTS; ++i) {
        if(l->p == p) {
          uip_unlisten(l->port);
          l->port = 0;
          l->p = PROCESS_NONE;
        }
        ++l;
      }

      {
        register struct uip_conn *cptr;

        for(cptr = &uip_conns[0]; cptr < &uip_conns[UIP_CONNS]; ++cptr) {
          if(cptr->appstate.p == p) {
            cptr->appstate.p = PROCESS_NONE;
            cptr->tcpstateflags = UIP_CLOSED;
          }

        }

      }
#endif /* UIP_TCP */
#if UIP_UDP
      {
        register struct uip_udp_conn *cptr;
        for(cptr = &uip_udp_conns[0];
            cptr < &uip_udp_conns[UIP_UDP_CONNS]; ++cptr) {
          if(cptr->appstate.p == p) {
            cptr->lport = 0;
          }
        }

      }
#endif /* UIP_UDP */
      break;

    case PROCESS_EVENT_TIMER:
      /* We get this event if one of our timers have expired. */
      {
        /* Check the clock so see if we should call the periodic uIP
           processing. */
        if(data == &periodic &&
           etimer_expired(&periodic)) {
#if UIP_TCP
          for(i = 0; i < UIP_CONNS; ++i) {
            if(uip_conn_active(i)) {
              /* Only restart the timer if there are active
                 connections. */
              etimer_restart(&periodic);
              uip_periodic(i);
              if(uip_len > 0) {
        		PRINTF("tcpip_output from periodic len %d\n", uip_len);
#if UIP6_CONF_IPV6
                if(uip_conn_ver(i)==IP_VER_6)
                    tcpip_ipv6_output();
                else
#endif /* UIP6_CONF_IPV6 */
                    tcpip_output();
        		PRINTF("tcpip_output after periodic len %d\n", uip_len);
              }
            }
          }
#endif /* UIP_TCP */
#if UIP_CONF_IP_FORWARD
          uip_fw_periodic();
#endif /* UIP_CONF_IP_FORWARD */
        }

#if UIP6_CONF_IPV6
#if UIP_CONF_IPV6_REASSEMBLY
        /*
         * check the timer for reassembly
         */
        if(data == &uip_reass_timer &&
           etimer_expired(&uip_reass_timer)) {
          uip_reass_over();
          tcpip_ipv6_output();
        }
#endif /* UIP_CONF_IPV6_REASSEMBLY */
        /*
         * check the different timers for neighbor discovery and
         * stateless autoconfiguration
         */
        /*if(data == &uip_nd6_timer_periodic &&
           etimer_expired(&uip_nd6_timer_periodic)) {
          uip_nd6_periodic();
          tcpip_ipv6_output();
        }*/
#if !UIP_CONF_ROUTER
        if(data == &uip_ds6_timer_rs && etimer_expired(&uip_ds6_timer_rs)){
          uip_ds6_send_rs();
          tcpip_ipv6_output();
        }
#endif /* !UIP_CONF_ROUTER */
        if(data == &uip_ds6_timer_periodic &&
           etimer_expired(&uip_ds6_timer_periodic)){
          uip_ds6_periodic();
          tcpip_ipv6_output();
        }
#endif /* UIP6_CONF_IPV6 */
      }
      break;

#if UIP_TCP
    case TCP_POLL:
      if(data != NULL) {
        uip_poll_conn(data);
        if(uip_len > 0) {
#if UIP6_CONF_IPV6
          if(uip_addr_ver(&(((struct uip_conn*)data)->ripaddr)) == IP_VER_6)
            tcpip_ipv6_output();
          else
#endif /* UIP6_CONF_IPV6 */
            tcpip_output();
     	  PRINTF("tcpip_output from tcp poll len %d\n", uip_len);
       }
        /* Start the periodic polling, if it isn't already active. */
        start_periodic_tcp_timer();
      }
      break;
#endif /* UIP_TCP */
#if UIP_UDP
    case UDP_POLL:
      if(data != NULL) {
        uip_udp_periodic_conn(data);
        if(uip_len > 0) {
#if UIP6_CONF_IPV6
          if(uip_addr_ver(&(((struct uip_udp_conn*)data)->ripaddr)) == 6)
            tcpip_ipv6_output();
          else
#endif /* UIP6_CONF_IPV6 */
            tcpip_output();
        }
      }
      break;
#endif /* UIP_UDP */

    case PACKET_INPUT:
      packet_input(data);
      break;
#if UIP6_CONF_IPV6
    case EV_ENABLE_IPV6:
      if(data != NULL)
      {
        int enable = *(int*)data;
        if(enable==UIP6_ENABLE)
        {
          PRINTF("**********Enable Ipv6!**********\r\n");
          uip_ds6_init();
        }
        else
        {
          PRINTF("**********Disable Ipv6!**********\r\n");
          uip_ds6_ipv6_disable();
        }
      }
      break;
    case START_SLAAC:
      etimer_restart(&uip_ds6_timer_rs);
      break;
#endif
  };
}
/*---------------------------------------------------------------------------*/
void
tcpip_input(int ip_v)
{
  process_data_t data = (process_data_t)&ip_v;
  process_post_synch(&tcpip_process, PACKET_INPUT, data);
  uip_len = 0;
#if UIP6_CONF_IPV6
  uip_ext_len = 0;
#endif /*UIP6_CONF_IPV6*/
}
/*---------------------------------------------------------------------------*/
#if UIP6_CONF_IPV6

void tcpip_start_send_rs(void)
{
    PRINTF("Start Sending RS Timer...\n");
    rscount=0;
    process_post(&tcpip_process, START_SLAAC, NULL);
}

void tcpip_enable_ipv6(int enable)
{
    uip6_ipv6_status = enable;
    if(enable == UIP6_DISABLE)
    {
        uip6_slaac_status = UIP6_DISABLE;
    }
    if(PROCESS_CURRENT() == 0)/*current process is null means during config file provision*/
    {
        process_post_synch(&tcpip_process, EV_ENABLE_IPV6, &uip6_ipv6_status);
    }
    else
    {
        process_post(&tcpip_process, EV_ENABLE_IPV6, &uip6_ipv6_status);
    }
}

void tcpip_enable_slaac(int enable)
{
    uip6_slaac_status = enable;
    if(UIP6_ENABLE == enable)
    {
        tcpip_start_send_rs();
    }
    else
    {
        uip_ds6_addr_rm(uip_ds6_get_slaac(-1));
        if(uip_ds6_defrt_list[0].isused == 1 &&
           uip_ds6_defrt_list[0].isinfinite == 0)
        {/*infinite is zero means defrt is configured by slaac,
           so when disable slaac, should delete related defrt*/
            uip_ds6_defrt_rm(&uip_ds6_defrt_list[0]);
        }
    }
}

int tcpip_ipv6_status()
{
    return uip6_ipv6_status;
}

int tcpip_slaac_status()
{
    return uip6_slaac_status;
}

void
tcpip_ipv6_output(void)
{
  uip_ds6_nbr_t *nbr = NULL;
  uip_ip6addr_t* nexthop;

  if(uip_len == 0) {
    return;
  }

  if(uip_len > UIP_LINK_MTU) {
    UIP_LOG("tcpip_ipv6_output: Packet to big");
    uip_len = 0;
    return;
  }
  if(uip_is_addr_unspecified(&UIP6_IP_BUF->destipaddr)){
    UIP_LOG("tcpip_ipv6_output: Destination address unspecified");
    uip_len = 0;
    return;
  }
  if(!uip_is_addr_mcast(&UIP6_IP_BUF->destipaddr)) {
    /* Next hop determination */
    nbr = NULL;
    if(uip_ds6_is_addr_onlink(&UIP6_IP_BUF->destipaddr)){
      nexthop = &UIP6_IP_BUF->destipaddr;
    } else {
      uip_ds6_route_t* locrt;
      locrt = uip_ds6_route_lookup(&UIP6_IP_BUF->destipaddr);
      if(locrt == NULL) {
        if((nexthop = uip_ds6_defrt_choose()) == NULL) {
#ifdef UIP_FALLBACK_INTERFACE
	  UIP_FALLBACK_INTERFACE.output();
#else
          PRINTF("tcpip_ipv6_output: Destination off-link but no route\n");
#endif
          uip_len = 0;
          return;
        }
      } else {
	nexthop = &locrt->nexthop;
      }
    }
    /* end of next hop determination */
    if((nbr = uip_ds6_nbr_lookup(nexthop)) == NULL) {
      //      printf("add1 %d\n", nexthop->u8[15]);
      if((nbr = uip_ds6_nbr_add(nexthop, NULL, 0, NBR_INCOMPLETE)) == NULL) {
        //        printf("add n\n");
        uip_len = 0;
        return;
      } else {
#if UIP_CONF_IPV6_QUEUE_PKT
        /* copy outgoing pkt in the queuing buffer for later transmmit */
        if(uip_packetqueue_alloc(&nbr->packethandle, UIP_DS6_NBR_PACKET_LIFETIME) != NULL) {
          memcpy(uip_packetqueue_buf(&nbr->packethandle), UIP6_IP_BUF, uip_len);
          uip_packetqueue_set_buflen(&nbr->packethandle, uip_len);
        }
#endif
      /* RFC4861, 7.2.2:
       * "If the source address of the packet prompting the solicitation is the
       * same as one of the addresses assigned to the outgoing interface, that
       * address SHOULD be placed in the IP Source Address of the outgoing
       * solicitation.  Otherwise, any one of the addresses assigned to the
       * interface should be used."*/
       if(uip_ds6_is_my_addr(&UIP6_IP_BUF->srcipaddr)){
          uip_nd6_ns_output(&UIP6_IP_BUF->srcipaddr, NULL, &nbr->ipaddr);
        } else {
          uip_nd6_ns_output(NULL, NULL, &nbr->ipaddr);
        }

        stimer_set(&(nbr->sendns), uip_ds6_if.retrans_timer / 1000);
        nbr->nscount = 1;
      }
    } else {
      if(nbr->state == NBR_INCOMPLETE) {
        PRINTF("tcpip_ipv6_output: nbr cache entry incomplete\n");
#if UIP_CONF_IPV6_QUEUE_PKT
        /* copy outgoing pkt in the queuing buffer for later transmmit and set
           the destination nbr to nbr */
        if(uip_packetqueue_alloc(&nbr->packethandle, UIP_DS6_NBR_PACKET_LIFETIME) != NULL) {
          memcpy(uip_packetqueue_buf(&nbr->packethandle), UIP6_IP_BUF, uip_len);
          uip_packetqueue_set_buflen(&nbr->packethandle, uip_len);
        }
        /*        memcpy(nbr->queue_buf, UIP_IP_BUF, uip_len);
                  nbr->queue_buf_len = uip_len;*/
        uip_len = 0;
#endif /*UIP_CONF_IPV6_QUEUE_PKT*/
        return;
      }
      /* if running NUD (nbc->state == STALE, DELAY, or PROBE ) keep
         sending in parallel see rfc 4861 Node behavior in section 7.7.3*/

      if(nbr->state == NBR_STALE) {
        nbr->state = NBR_DELAY;
        stimer_set(&(nbr->reachable),
                  UIP_ND6_DELAY_FIRST_PROBE_TIME);
        nbr->nscount = 0;
        PRINTF("tcpip_ipv6_output: nbr cache entry stale moving to delay\n");
      }

      stimer_set(&(nbr->sendns),
                uip_ds6_if.retrans_timer / 1000);

      tcpip_output6(&(nbr->lladdr));


#if UIP_CONF_IPV6_QUEUE_PKT
      /* Send the queued packets from here, may not be 100% perfect though.
       * This happens in a few cases, for example when instead of receiving a
       * NA after sendiong a NS, you receive a NS with SLLAO: the entry moves
       *to STALE, and you must both send a NA and the queued packet
       */
      /*      if(nbr->queue_buf_len != 0) {
        uip_len = nbr->queue_buf_len;
        memcpy(UIP_IP_BUF, nbr->queue_buf, uip_len);
        nbr->queue_buf_len = 0;
        tcpip_output(&(nbr->lladdr));
        }*/
      if(uip_packetqueue_buflen(&nbr->packethandle) != 0) {
        uip_len = uip_packetqueue_buflen(&nbr->packethandle);
        memcpy(UIP6_IP_BUF, uip_packetqueue_buf(&nbr->packethandle), uip_len);
        uip_packetqueue_free(&nbr->packethandle);
        tcpip_output6(&(nbr->lladdr));
      }
#endif /*UIP_CONF_IPV6_QUEUE_PKT*/

      uip_len = 0;
      return;
    }
  }

  /*multicast IP destination address */
  tcpip_output6(NULL);
  uip_len = 0;
  uip_ext_len = 0;

}
#endif
/*---------------------------------------------------------------------------*/
#if UIP_UDP
void
tcpip_poll_udp(struct uip_udp_conn *conn)
{
  process_post(&tcpip_process, UDP_POLL, conn);
}
#endif /* UIP_UDP */
/*---------------------------------------------------------------------------*/
#if UIP_TCP
void
tcpip_poll_tcp(struct uip_conn *conn)
{
  process_post(&tcpip_process, TCP_POLL, conn);
}
#endif /* UIP_TCP */
/*---------------------------------------------------------------------------*/
void
tcpip_uipcall(void)
{
  register uip_udp_appstate_t *ts;

#if UIP_UDP
  if(uip_conn != NULL) {
    ts = &uip_conn->appstate;
  } else {
    ts = &uip_udp_conn->appstate;
  }
#else /* UIP_UDP */
  ts = &uip_conn->appstate;
#endif /* UIP_UDP */

#if UIP_TCP
 {
   static unsigned char i;
   register struct listenport *l;

   /* If this is a connection request for a listening port, we must
      mark the connection with the right process ID. */
   if(uip_connected()) {
     l = &s.listenports[0];
     for(i = 0; i < UIP_LISTENPORTS; ++i) {
       if(l->port == uip_conn->lport &&
	  l->p != PROCESS_NONE) {
	 ts->p = l->p;
	 ts->state = NULL;
	 break;
       }
       ++l;
     }

     /* Start the periodic polling, if it isn't already active. */
     start_periodic_tcp_timer();
   }
 }
#endif /* UIP_TCP */

  if(ts->p != NULL) {
    process_post_synch(ts->p, tcpip_event, ts->state);
  }
}
/*---------------------------------------------------------------------------*/
PROCESS_THREAD(tcpip_process, ev, data)
{
  PROCESS_BEGIN();

#if UIP_TCP
 {
   static unsigned char i;

   for(i = 0; i < UIP_LISTENPORTS; ++i) {
     s.listenports[i].port = 0;
   }
   s.p = PROCESS_CURRENT();
 }
#endif

  tcpip_event = process_alloc_event();
#if UIP_CONF_ICMP6
  tcpip_icmp6_event = process_alloc_event();
#endif /* UIP_CONF_ICMP6 */
  etimer_set(&periodic, CLOCK_SECOND / 2);

  uip_init();
#ifdef UIP_FALLBACK_INTERFACE
  UIP_FALLBACK_INTERFACE.init();
#endif
/* initialize RPL if configured for using RPL */
#if UIP_CONF_IPV6_RPL
  rpl_init();
#endif /* UIP_CONF_IPV6_RPL */

  while(1) {
    PROCESS_YIELD();
    eventhandler(ev, data);
  }

  PROCESS_END();
}
/*---------------------------------------------------------------------------*/
