/* 
 * Copyright (c) 2012 Qualcomm Atheros, Inc.. 
 * All Rights Reserved. 
 * Qualcomm Atheros Confidential and Proprietary. 
 */

#ifndef __MUSIC_SPI_H__
#define __MUSIC_SPI_H__

#ifdef __cplusplus
extern "C" {
#endif    /* __cplusplus */


typedef unsigned int uint32_t;
typedef unsigned char uint8_t;

#define SPI_FLASH 1
#define MUSIC_GPIO_OE          0x18040000   /*gpio output enable */
#define MUSIC_SPI_DSEL          0xb8000008  /*gpio output enable */
#define MUSIC_SPI_FS           0x19000000   /*SPI FUCTION SELECT REG */
#define MUSIC_SPI_CLOCK        0x19000004   /*SPI  CONTROL REG */
#define MUSIC_SPI_WRITE        0x19000008   /*SPI  IO CONTROL REG */
#define MUSIC_SPI_READ         0x19000000   /*SPI  READ DATA REG */
#define MUSIC_SPI_RD_STATUS    0x1900000c   /*no need more */

#define MUSIC_SPI_CS_DIS       0x70000
#define MUSIC_SPI_CE_LOW       0x60000
#define MUSIC_SPI_CE_HIGH      0x60100

#define MUSIC_SPI_CMD_WREN         0x06
#define MUSIC_SPI_CMD_RD_STATUS    0x05
#define MUSIC_SPI_CMD_FAST_READ    0x0b
#define MUSIC_SPI_CMD_PAGE_PROG    0x02
//#define MUSIC_SPI_CMD_SECTOR_ERASE 0xd8
#define MUSIC_SPI_CMD_SECTOR_ERASE 0x20
#define MUSIC_SPI_CMD_BULK_ERASE   0xc7
#define MUSIC_SPI_CMD_RDID         0x9f


//#define MUSIC_SPI_SECTOR_SIZE      (1024*256)
#define MUSIC_SPI_SECTOR_SIZE       0x1000
#define MUSIC_SPI_PAGE_SIZE        256

/*#define delay(val) do {        \
    int i;\
    for(i=val;i>0;i--);\
}while(0);*/

#define SPI_MX25L25635E_ID      0xc22019
#define SPI_MX25L6406E_ID       0xc22017
#define SPI_M25Q128_ID          0x12018
#define SPI_M25P64_ID           0x202017
#define SPI_M25P128_ID          0x202018
#define SPI_MX25L128_ID         0xc22018

typedef struct board_spi_flash_info
{
    uint32_t flash_id;
    uint32_t flash_size;
} board_spi_flash_info_t;

typedef struct flash_segment_info
{
    uint32_t segment_addr;
    uint32_t segment_size;
} flash_segment_info_t;

typedef struct board_partition_info
{
    uint32_t flash_size;
    flash_segment_info_t loader;
    flash_segment_info_t control_block;
    flash_segment_info_t image_a;
    flash_segment_info_t image_b;
    flash_segment_info_t running_image;
    flash_segment_info_t boot_cfg;
    flash_segment_info_t user_cfg;
} board_partition_info_t;


uint32_t addr_change(uint32_t _phys);

#define KSEG1ADDR(_x)   (0xa0000000 | (_x))

#define music_reg_rd(_phys)    (*(volatile uint32_t *)KSEG1ADDR(_phys))

#define music_reg_wr(_phys, _val) do {     \
    ((*(volatile uint32_t *)KSEG1ADDR(_phys)) = (_val)); \
}while(0);

/*
 * primitives
 */

#define music_be_msb(_val, _i) (((_val) & (1 << (7 - _i))) >> (7 - _i))

#define music_spi_bit_banger(_byte)  do {        \
    int i;    \
    for(i = 0; i < 8; i++) {                    \
        music_reg_wr(MUSIC_SPI_WRITE,      \
                        MUSIC_SPI_CE_LOW | music_be_msb(_byte, i));  \
        music_reg_wr(MUSIC_SPI_WRITE,      \
                        MUSIC_SPI_CE_HIGH | music_be_msb(_byte, i)); \
    }       \
}while(0);

#define music_spi_go() do {        \
    music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CE_LOW); \
    music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CS_DIS); \
}while(0);


#define music_spi_send_addr(_addr) do {                    \
    music_spi_bit_banger(((_addr & 0xff0000) >> 16));                 \
    music_spi_bit_banger(((_addr & 0x00ff00) >> 8));                 \
    music_spi_bit_banger(_addr & 0x0000ff);                 \
}while(0);

#define music_spi_delay_8()    music_spi_bit_banger(0)
void spiflash_sector_erase(uint32_t addr);
void spiflash_write_page(uint32_t addr, uint8_t * data, int len);


#ifdef __cplusplus
}
#endif    /* __cplusplus */

#endif                          /* __MUSIC_SPI_H__ */
