
#ifndef __UBOOT_MUSIC_SWITCH_H__
#define __UBOOT_MUSIC_SWITCH_H__

#ifdef __cplusplus
extern "C" {
#endif    /* __cplusplus */


#define MUSIC_SWITCH_BASE                           0x18800000
#define REG_GLOBAL_CTRL_0                           0
#define REG_LED_GOL_CTRL_1                          0x14004

#define MUSIC_LED_GROUP_LENGTH1                     0
#define MUSIC_LED_GROUP_LENGTH2                     1
#define MUSIC_LED_GROUP_LENGTH4                     2

#define MUSIC_LED_GROUP_INDEX0                      0
#define MUSIC_LED_GROUP_INDEX1                      1
#define MUSIC_LED_GROUP_INDEX2                      2
#define MUSIC_LED_GROUP_INDEX3                      3
#define MUSIC_LED_GROUP_INDEX4                      4
#define MUSIC_LED_GROUP_INDEX5                      5
#define MUSIC_LED_GROUP_INDEX6                      6

#define REG_LED_COL_CTRL_1_LED_GROUP012_MASK        (3<<8)
#define REG_LED_COL_CTRL_1_LED_GROUP012_SHIFT       8
#define REG_LED_COL_CTRL_1_LED_GROUP3_MASK          (3<<10)
#define REG_LED_COL_CTRL_1_LED_GROUP3_SHIFT         10
#define REG_LED_COL_CTRL_1_LED_GROUP4_MASK          (3<<12)
#define REG_LED_COL_CTRL_1_LED_GROUP4_SHIFT         12
#define REG_LED_COL_CTRL_1_LED_GROUP5_MASK          (3<<14)
#define REG_LED_COL_CTRL_1_LED_GROUP5_SHIFT         14
#define REG_LED_COL_CTRL_1_LED_GROUP6_MASK          (3<<16)
#define REG_LED_COL_CTRL_1_LED_GROUP6_SHIFT         16

#define GLOBAL_CTRL_0                               0
#define GLOBAL_CTRL_0_SOFT_RST_MASK                 0x80000000
#define GLOBAL_CTRL_0_REG_RST_EN_MASK               0x00040000

#define REG_MAC_XMII_CTRL                           0x0008
#define MAC_XMII_PHY_RST_EN_CFG_MASK                0x01000000

#define REG_PORT_STATUS	                            0x100

#define REG_GLOBAL_CTRL_1                           8

#define     SGMII_INTERFACE_TYPE                    0
#define     QSGMII_INTERFACE_TYPE                   1

#define GLOBAL_CTRL_1_MAC29SG_1P25G_EN_MASK         (1<<21)
#define GLOBAL_CTRL_1_MAC29SG_3P125G_EN_MASK        (1<<20)
#define GLOBAL_CTRL_1_MAC28SG_1P25G_EN_MASK         (1<<19)
#define GLOBAL_CTRL_1_MAC28SG_3P125G_EN_MASK        (1<<18)

#define REG_SGMII_CTRL0                             0x13100
#define SGMII_MODE_1000BASEX                        0
#define SGMII_MODE_PHY                              1
#define SGMII_MODE_MAC                              2
#define SGMII_CTRL0_MODE_CTRL_MASK                  0xc00000
#define SGMII_CTRL0_MODE_CTRL_SHIFT                 22


#define REG_XAUI_SERDES13_CTRL0                     0x1341c
#define XAUI_SERDES13_CTRL0_XAUI_EN_SD_MASK         (0xf << 21)

#define REG_XAUI_SERDES13_CTRL1                     0x13420
#define XAUI_SERDES13_CTRL1_XAUI_EN_RX              (0xf << 6)
#define XAUI_SERDES13_CTRL1_XAUI_EN_TX              (0xf << 10)

#define REG_SGMII_SERDES7_CTRL0                     0x1340c
#define SGMII_SERDES7_CTRL0_SGMII_EN_TX_MASK        (3<<27)
#define SGMII_SERDES7_CTRL0_SGMII_EN_TX_SHIFT       27
#define SGMII_SERDES7_CTRL0_SGMII_EN_RX_MASK        (3<<21)
#define SGMII_SERDES7_CTRL0_SGMII_EN_RX_SHIFT       21
#define SGMII_SERDES7_CTRL0_SGMII_EN_PLL_MASK       2


#define music_switch_wr(addr, val) music_reg_wr(MUSIC_SWITCH_BASE + addr, val)
#define music_switch_rd(addr) music_reg_rd(MUSIC_SWITCH_BASE + addr)

extern int32_t music_switch_led_group_enable(uint32_t group_index, uint32_t enable);


extern int32_t music_switch_led_length_set(uint32_t group_index, uint32_t led_length);

extern int32_t music_switch_uplink_port_enable(void);

extern void music_switch_soft_reset(void);


extern int32_t music_switch_port2324_interface_select(uint32_t port_id, uint32_t interface_type);

extern int32_t music_switch_sgmii_mode_type_set(uint32_t port_id, uint32_t sgmii_mode);

extern int32_t music_switch_slave_port2324_interface_select(uint32_t port_id, uint32_t interface_type);

extern int32_t music_switch_slave_sgmii_mode_type_set(uint32_t port_id, uint32_t sgmii_mode);

extern int32_t music_switch_sgmii_init(void);

extern int32_t music_switch_slave_sgmii_init(void);


#ifdef __cplusplus
}
#endif    /* __cplusplus */

#endif
