/*
 * Copyright (c) 2012 Qualcomm Atheros, Inc..
 * All Rights Reserved.
 * Qualcomm Atheros Confidential and Proprietary.
 */

#ifndef __SPIFLASH_H__
#define __SPIFLASH_H__

#ifdef __cplusplus
extern "C" {
#endif    /* __cplusplus */


#define MUSIC_GPIO_OE          0x18040000   /*gpio output enable */
#define MUSIC_SPI_DSEL          0xb8000008  /*gpio output enable */
#define MUSIC_SPI_FS           0x19000000   /*SPI FUCTION SELECT REG */
#define MUSIC_SPI_CLOCK        0x19000004   /*SPI  CONTROL REG */
#define MUSIC_SPI_WRITE        0x19000008   /*SPI  IO CONTROL REG */
#define MUSIC_SPI_READ         0x19000000   /*SPI  READ DATA REG */
#define MUSIC_SPI_RD_STATUS    0x1900000c   /*no need more */

#define MUSIC_SPI_CS_DIS       0x70000
#define MUSIC_SPI_CE_LOW       0x60000
#define MUSIC_SPI_CE_HIGH      0x60100

#define MUSIC_SPI_CMD_WREN         0x06
#define MUSIC_SPI_CMD_RD_STATUS    0x05
#define MUSIC_SPI_CMD_FAST_READ    0x0b
#define MUSIC_SPI_CMD_PAGE_PROG    0x02
#ifdef QFN_DEMO_SPI
#define MUSIC_SPI_CMD_SECTOR_ERASE 0x20
#else
#define MUSIC_SPI_CMD_SECTOR_ERASE 0xd8
#endif
#define MUSIC_SPI_CMD_BULK_ERASE   0xc7
#define MUSIC_SPI_CMD_RDID         0x9f


#ifndef CUSTOM_DEF_SPI
#define MUSIC_SPI_SECTOR_SIZE      0x40000
#define MUSIC_SPI_PAGE_SIZE        0x100
#else
#ifdef QFN_DEMO_SPI
#define MUSIC_SPI_SECTOR_SIZE      0x1000
#define MUSIC_SPI_PAGE_SIZE        0x100
#endif
#endif


uint32_t addr_change(uint32_t _phys);

/*
 * primitives
 */

#define music_be_msb(_val, _i) (((_val) & (1 << (7 - _i))) >> (7 - _i))

#define music_spi_bit_banger(_byte)  do {        \
    int i;    \
    for(i = 0; i < 8; i++) {                    \
        music_reg_wr(MUSIC_SPI_WRITE,      \
                        MUSIC_SPI_CE_LOW | music_be_msb(_byte, i));  \
        music_reg_wr(MUSIC_SPI_WRITE,      \
                        MUSIC_SPI_CE_HIGH | music_be_msb(_byte, i)); \
    }       \
}while(0);

#define music_spi_go() do {        \
    music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CE_LOW); \
    music_reg_wr(MUSIC_SPI_WRITE, MUSIC_SPI_CS_DIS); \
}while(0);


#define music_spi_send_addr(_addr) do {                    \
    music_spi_bit_banger(((addr & 0xff0000) >> 16));                 \
    music_spi_bit_banger(((addr & 0x00ff00) >> 8));                 \
    music_spi_bit_banger(addr & 0x0000ff);                 \
}while(0);

#define music_spi_delay_8()    music_spi_bit_banger(0)


#ifdef __cplusplus
}
#endif    /* __cplusplus */

#endif
