/* * Copyright (c) 2012 Qualcomm Atheros, Inc.
 * All Rights Reserved. 
 *Qualcomm Atheros Confidential and Proprietary. 
 *Notifications and licenses are retained for attribution purposes only */

/*
 *  Startup Code for MIPS32 CPU-core
 *
 *  Copyright (c) 2003	Wolfgang Denk <wd@denx.de>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


#include <asm/regdef.h>
#include <asm/mipsregs.h>

#define CFG_SRAM_BASE 0xbd000000


.set noreorder

.globl _start
.text

_start:

#if 1
	/*save old ra*/
	li	t0, CFG_SRAM_BASE + 0x7FF0
	sw	ra,	0(t0)

    /*save old sp*/
	li	t0, CFG_SRAM_BASE + 0x7FF0 - 4
	sw	sp,	0(t0)

	/*set new sp*/
	li	t0, CFG_SRAM_BASE + 0x7FF0 - 8
	la	sp, 0(t0)
#else
	li	t0, 0xbd000000 + 0x1ff0
	la	sp, 0(t0)
#endif


	bal     1f
	nop
	.word	0x00000000	/* See the above e-mail */
	/*
	 * Clearing CP0 registers - This is generally required for the MIPS-24k
	 * core used by Atheros.
	 */
1:
	la	t0, rel_start
	j	t0
	nop

rel_start:


	/* Jump to clear BSS
	 */
	la	t0, clear_bss
	j	t0
	nop

	.word	boot_end_data
	.word	boot_end

clear_bss:

	/* Clear BSS.
	 */
	lw	t1, -8(t0)	/* t1 <-- boot_end_data	*/
	lw	t2, -4(t0)	/* t2 <-- boot_end		*/
	//add	t1, t6		/* adjust pointers		*/
	//add	t2, t6

	sub	t1, 4
1:	addi	t1, 4
	bltl	t1, t2, 1b
	sw	zero, 0(t1)	/* delay slot			*/


	la	t9, main
	j	t9
	nop

/*
 * Cannot trust the first stage f/w. Don't know what registers
 * it saves or clobbers. Save them and call the f/w and restore
 * them on return. PCIe-EP and USB didn't have any problem.
 * However, when called from nand boot-rom, there were issues,
 * hence the paranoia
 *
 * Refer: http://math-atlas.sourceforge.net/devel/assembly/mipsabi32.pdf
 *
 * Only registers $16..$23 and registers $28.$30 are preserved across
 * a function call. Register $28 is not preserved, however, when
 * calling position independent code.
 *
 *
 * void call_fw(arg, func)
 */
#define	reg_space	(13 * 4)	/* 11 regs + 1 signature */

	.globl	call_fw
	.text

call_fw:
	addiu	sp,	(reg_space * -1)	/* space for registers */
	sw	s0,	4(sp)			/* Save them */
	sw	s1,	8(sp)
	sw	s2,	12(sp)
	sw	s3,	16(sp)
	sw	s4,	20(sp)
	sw	s5,	24(sp)
	sw	s6,	28(sp)
	sw	s7,	32(sp)
	sw	s8,	36(sp)
	sw	gp,	40(sp)
	sw	ra,	44(sp)

#define detect_stack_corruption
#ifdef detect_stack_corruption
	lui	t0,	0x7761			/* wasp == 0x77617370 */
	ori	t0,	0x7370

	sw	t0,	0(sp)			/* sign the top of stack */
#endif /* detect_stack_corruption */

	jalr	a1				/* call the function */
	nop					/* delay slot */

#ifdef detect_stack_corruption
	lw	t1,	0(sp)			/* check the signature */

	lui	t0,	0x7761
	ori	t0,	0x7370

stack_corrupted:
	bne	t1,	t0,	stack_corrupted
	nop					/* delay slot */
#endif /* detect_stack_corruption */

	lw	s0,	4(sp)			/* Restore them */
	lw	s1,	8(sp)
	lw	s2,	12(sp)
	lw	s3,	16(sp)
	lw	s4,	20(sp)
	lw	s5,	24(sp)
	lw	s6,	28(sp)
	lw	s7,	32(sp)
	lw	s8,	36(sp)
	lw	gp,	40(sp)
	lw	ra,	44(sp)

	addiu	sp,	reg_space		/* Unwind stack */

	j	ra				/* return */
	nop					/* delay slot */

	/* Exception handlers.
	 */
romReserved:
	b romReserved

romExcHandle:
	b romExcHandle

	.globl	main_end
	.text

main_end:
	/*restore old ra*/
	li	t0, CFG_SRAM_BASE + 0x7FF0
	lw	ra,	0(t0)

    /*restore old sp*/
	li	t0, CFG_SRAM_BASE + 0x7FF0 - 4
	lw	sp,	0(t0)

    j	ra			
    nop	

