#include "mipsregs.h"
#include "music_spi.h"
#include "music_upgrade.h"

typedef void *cmd_function_t(void);

typedef struct
{
    uint8_t *cmd_name_p;
    cmd_function_t *cmd_fun_p;
    uint8_t *cmd_explain_p;
} loader_cmd_t;

void my_test_function(void);
void boot_elite(void);
void uncomp(void);
void print_cmd(void);
void ctrl_block_A_correct();
void ctrl_block_B_correct();
void ctrl_block_default();
void control_block_dump();
void image_upgrade(void);
void image_upgrade_done(void);
void boot_in_loader(void);
void elite_reset(void);
void sys_boot(void);

loader_cmd_t loader_commands[] = {
#if 0
    {"test", &my_test_function, "test command"},
    {"uncomp", uncomp, "umcompress image"},
    {"boot_only", boot_elite, "only boot(without looking into ctrl block info)"},
    {"boot_in_ld", boot_in_loader, "boot elite in loader"},
    {"cor_a", ctrl_block_A_correct, "set imageA into correct"},
    {"cor_b", ctrl_block_B_correct, "set imageB into correct"},
    {"ctrl_def", ctrl_block_default, "set default ctrl block info"},
    {"up_ctrl", image_upgrade, "upgrade image flag"},
    {"up_done", image_upgrade_done, "simulate image upgrade done in firmware"},
#endif
    {"boot", sys_boot, "boot elite command"},
    {"dump_ctrl", control_block_dump, "dump ctrl info"},
    {"help", print_cmd, "help command"},
    {"reset", elite_reset, "reset elite command"}
};

#define COMMAND_COUNT sizeof(loader_commands)/sizeof(loader_cmd_t)

void command_handler(uint8_t * cmd_name)
{
    uint32_t i = 0;
    //printf("\ncmd:%s len:%d COMMAND_COUNT:%d\n", cmd_name, strlen(cmd_name), COMMAND_COUNT);
    for (i = 0; i < COMMAND_COUNT; i++)
    {
        if (strcmp(cmd_name, loader_commands[i].cmd_name_p) == 0)
        {
            printf("\n\r");
            loader_commands[i].cmd_fun_p();
            return;
        }

    }
    return;
}

void my_test_function(void)
{
    printf("just for test\n\r");
}

void uncomp(void)
{
    if (whether_image_is_decompress() == 0)
    {
        if (uncompress_image(IMAGE_RUNNING_OFFSET, image_compress_offset()) == 0)
        {
            printf("decompressing image is successful\n\r");
        }
    }
}



void boot_elite(void)
{
    watchdog_init();
    watchdog_start();
    music_boot_elite();
}

void sys_boot(void)
{

    uint32_t image_physical_base = boot_image_addr_computer();

    if (image_physical_base == IMAGE_A_OFFSET)
        printf("bootloader select image A!\n\r");
    else if (image_physical_base == IMAGE_B_OFFSET)
        printf("bootloader select image B!\n\r");
    else
    {
        printf("some serious problems occures, back into command line in loader\n\r");
        return;
    }

    if (whether_image_is_decompress() == 0)
    {
        if (uncompress_image(IMAGE_RUNNING_OFFSET, image_physical_base) == 0)
        {
            printf("decompressing image is successful\n\r");
        }
        else
        {
            printf("decompressing image is failed\n\r");
            return;
        }
    }

    boot_elite();
}


void print_cmd(void)
{
    uint32_t i;
    printf("\n\r");
    for (i = 0; i < COMMAND_COUNT; i++)
    {
        printf("%s:%s\n", loader_commands[i].cmd_name_p, loader_commands[i].cmd_explain_p);
    }
}

void boot_in_loader(void)
{
    printf("boot phy addr:0x%x\n\r", boot_image_addr_computer());
}

void elite_reset()
{
    printf("******************* System Reset ********************\n\r");

    music_reg_wr(0x18000000, 0x80000000);
    while (1)
    {
    }

}
