/*
 * Copyright (c) 2012 Qualcomm Atheros, Inc..
 * All Rights Reserved.
 * Qualcomm Atheros Confidential and Proprietary.
 */

#include "music_spi.h"
#include "music_upgrade.h"
#include "mipsregs.h"
#include "music_util.h"

#define FAILED  -1

#define DEBUG 0
#if DEBUG
#define debug_upgrade_printf(...)    printf(__VA_ARGS__)
#else
#define debug_upgrade_printf(...)
#endif

#if TODO
#define call_sync()                                     \
	__asm__ __volatile__(                                   \
		".set noreorder\n\t"                            \
		"sync\n\t"                                     \
".set reorder")

#define tlb_read_indexed()                                     \
	__asm__ __volatile__(                                   \
		".set noreorder\n\t"                            \
		"tlbr\n\t"                                     \
".set reorder")
#endif
//extern uint32_t running_seconds;
int printf(const char *fmt, ...);
int memcmp(const void *buf1, const void *buf2, unsigned int count);
int strcmp(const char *s1, const char *s2);
uint32_t image_header_check(image_header_t * image_header)
{
//TODO
#if 0
    board_info_t *board_info = (board_info_t *) BOARD_INFO_BASE;
    if (strcmp(image_header->magic, (char *)"elite image") ||
        strcmp(image_header->oem_id, board_info->oem_id) || strcmp(image_header->oem_id, board_info->oem_id))
        return (FAILED);
#endif
    if (image_header->image_header_length != IMAGE_HEADER_SIZE)
    {
        printf("image_header_length:0x%x != IMAGE_HEADER_SIZE:0x%x\n\r",
               image_header->image_header_length, IMAGE_HEADER_SIZE);
        return (FAILED);
    }
    if (image_header->image_length > IMAGE_MAX_LENGTH)
    {
        printf("image_length:0x%x exceed IMAGE_MAX_LENGTH:0x%x\n\r", image_header->image_length, IMAGE_MAX_LENGTH);
        return (FAILED);
    }
    if (image_header->image_length % 4 != 0)
    {
        printf("image_length:0x%x should be devided by 4\n\r", image_header->image_length);
        return (FAILED);
    }
    return 0;
}

#if TODO
#define call_sync()                                     \
	__asm__ __volatile__(                                   \
		".set noreorder\n\t"                            \
		"sync\n\t"                                     \
".set reorder")
#endif
uint32_t current_running_image()
{
#if TODO

#if 0
    ctrl_info_t *ctrl_info = (ctrl_info_t *) CONTROL_BLOCK_BASE;
    if (ctrl_info->upgrade_flag == UPGRADE_NSET)
        return ctrl_info->select_flag;
    else if (ctrl_info->upgrade_flag == UPGRADE_SET)
        return ctrl_info->will_select_flag;
#else
    uint32_t image_physical_address, start_addr;
    write_32bit_cp0_register(MIPS_CP0_INDEX, 0);
    call_sync();
    tlb_read_indexed();
    call_sync();
    start_addr = (read_32bit_cp0_register(MIPS_CP0_ENTRYLO0) & 0xffffffc0) << 6;
    image_physical_address = KSEG1ADDR(start_addr);
    debug_upgrade_printf("image_physical_address:0x%x\n\r", image_physical_address);

    if ((image_physical_address >= IMAGE_A_BASE) && (image_physical_address < IMAGE_A_BASE + IMAGE_MAX_LENGTH))
        return IMAGE_A;
    else if ((image_physical_address >= IMAGE_B_BASE) && (image_physical_address < IMAGE_B_BASE + IMAGE_MAX_LENGTH))
        return IMAGE_B;
    else
        printf("!!!!!!!!!we should not go here!!!!!!!\n\r");
    return FAILED;
#endif
#endif
    return IMAGE_A;
}

#if TODO
uint32_t image_crc_check(uint32_t which_image)
{
    uint32_t len = 0;
    char *image_addr;
    char checksum = 0;
    image_header_t *image_header;
    uint32_t upgrade_image_base;

    if ((which_image != IMAGE_A) && (which_image != IMAGE_B))
    {
        printf("input which_image is not correct\n\r");
        return FAILED;
    }

    upgrade_image_base = (which_image == IMAGE_A) ? IMAGE_A_BASE : IMAGE_B_BASE;
    image_header = (image_header_t *) upgrade_image_base;

    if (image_header_check(image_header))
    {
        debug_upgrade_printf("%s head crc check failed!\n\r", (which_image == IMAGE_A) ? "IMAGE_A" : "IMAGE_B");
        return FAILED;
    }

    image_addr = (char *)(upgrade_image_base + IMAGE_HEADER_SIZE);
    debug_upgrade_printf("image_addr:0x%x, image_length:0x%x, image_checksum:0x%x\n\r",
                         (uint32_t) image_addr, image_header->image_length, image_header->image_checksum);
    len = 0;
    while (len < image_header->image_length)
    {
        checksum += *image_addr;
        image_addr++;
        len++;
    }
    if (image_header->image_checksum != checksum)
    {
        printf("computed checksum:0x%x != image_checksum:0x%x\n\r", checksum, image_header->image_checksum);
        return FAILED;
    }
    return 0;
}

uint32_t control_block_crc_compute(ctrl_info_t * control_info)
{
    uint32_t checksum, length, i;
    uint32_t *checksum_p = (uint32_t *) control_info;
    checksum = 0;
    length = CONTROL_BLOCK_LEN / 4 - 1;
    for (i = 0; i < length; i++)
    {
        checksum += *checksum_p;
        checksum_p++;
    }
    return checksum;
}

uint32_t control_block_verification(ctrl_info_t * control_info)
{
    uint32_t checksum, length, i;
    uint32_t *checksum_p = (uint32_t *) control_info;
    checksum = 0;
    length = CONTROL_BLOCK_LEN / 4 - 1;
    debug_upgrade_printf("control block@0x%x length:%d\n\r", (uint32_t) control_info, length);
    for (i = 0; i < length; i++)
    {
        checksum += *checksum_p;
        checksum_p++;
    }
    if (checksum != control_info->crc_32bit)
    {
        debug_upgrade_printf("control block@0x%x crc error!\n\r", (uint32_t) control_info);
        return FAILED;
    }
    else
        return 0;
}

uint32_t control_block_copy(ctrl_info_t * dest, ctrl_info_t * source)
{
    uint32_t dest_addr, src_addr;
    char temp_buf[MUSIC_SPI_PAGE_SIZE];
    dest_addr = (uint32_t) dest;
    src_addr = (uint32_t) source;
    if (!((dest_addr == CONTROL_BLOCK_BASE) || (dest_addr == CONTROL_BLOCK_BACKUP1_ADDR)
          || (dest_addr == CONTROL_BLOCK_BACKUP2_ADDR)))
    {
        printf("dest@0x%x is not correct\n\r", dest_addr);
        return FAILED;
    }
    if ((uint32_t) dest == (uint32_t) source)
    {
        printf("dest equals to source address!\n\r");
        return FAILED;
    }

    spiflash_sector_erase(dest_addr - SPI_FLASH_BASE);
    if ((src_addr == CONTROL_BLOCK_BASE) || (src_addr == CONTROL_BLOCK_BACKUP1_ADDR)
        || (src_addr == CONTROL_BLOCK_BACKUP2_ADDR))
    {
        debug_upgrade_printf("need to copy src_buf into memory\n\r");
        memcpy(temp_buf, (uint8_t *) (src_addr), MUSIC_SPI_PAGE_SIZE);
        spiflash_write_page(dest_addr - SPI_FLASH_BASE, (uint8_t *) (temp_buf), sizeof(*source));
    }
    else
        spiflash_write_page(dest_addr - SPI_FLASH_BASE, (uint8_t *) (src_addr), sizeof(*source));

    return 0;
}

uint32_t control_block_backup_check()
{
    ctrl_info_t *backup1, *backup2, *source, *dest;
    backup1 = (ctrl_info_t *) CONTROL_BLOCK_BACKUP1_ADDR;
    backup2 = (ctrl_info_t *) CONTROL_BLOCK_BACKUP2_ADDR;
    if (control_block_verification(backup1))
    {
        debug_upgrade_printf("backup1 crc error\n\r");
        source = backup2;
        dest = backup1;
        if (control_block_verification(backup2))
        {
            printf("both backup1 and backup2 crc error\n\r");
            return FAILED;
        }
    }
    else if (control_block_verification(backup2))
    {
        debug_upgrade_printf("backup2 crc error\n\r");
        source = backup1;
        dest = backup2;
    }
    else
    {
        //check whether backup1 and backup2 is the same. If not, use backup1 to restore backup2
        if (memcmp((void *)CONTROL_BLOCK_BACKUP1_ADDR, (void *)CONTROL_BLOCK_BACKUP2_ADDR, sizeof(ctrl_info_t)))
        {
            printf("backup1 and backup2 is not the same, but crc correct\n\r");
            source = backup1;
            dest = backup2;
        }
        else
            return 0;
    }

    if (control_block_copy(dest, source))
    {
        printf("control_block_copy failed!\n\r");
        return FAILED;
    }

    debug_upgrade_printf("recover control block from backup info\n\r");
    control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BASE, source);
    return 0;
}

uint32_t control_block_backup_save()
{
    if (control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BACKUP1_ADDR, (ctrl_info_t *) CONTROL_BLOCK_BASE))
    {
        debug_upgrade_printf("copy CONTROL_BLOCK_BACKUP1_ADDR failed\n\r");
        return FAILED;
    }
    if (control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BACKUP2_ADDR, (ctrl_info_t *) CONTROL_BLOCK_BASE))
    {
        debug_upgrade_printf("copy CONTROL_BLOCK_BACKUP2_ADDR failed\n\r");
        return FAILED;
    }
    return 0;
}

uint32_t image_upgrade_identification_set()
{
    ctrl_info_t control_block;
    uint32_t image;
    uint32_t current_image = current_running_image();
    image = (current_image == IMAGE_A) ? IMAGE_B : IMAGE_A;

    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BASE, sizeof(control_block));
    control_block.upgrade_flag = UPGRADE_SET;
    control_block.will_select_flag = image;
    if (image == IMAGE_A)
    {
        control_block.imageA_loader_count = 0;
        control_block.imageA_valid_flag = IMAGE_UNVALID;
    }
    else
    {
        control_block.imageB_loader_count = 0;
        control_block.imageB_valid_flag = IMAGE_UNVALID;
    }
    control_block.crc_32bit = control_block_crc_compute(&control_block);
    debug_upgrade_printf("control_block@0x%x crc_32bit:0x%x@0x%x\n\r", (uint32_t) & control_block,
                         control_block.crc_32bit, (uint32_t) & control_block.crc_32bit);
    return control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BASE, (ctrl_info_t *) & control_block);
}

uint32_t image_upgrade_identification_check()
{
    ctrl_info_t control_block;
    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BASE, sizeof(control_block));
    if (control_block_verification(&control_block))
    {
        printf("control block crc error, this should not happen!\n\r");
        return FAILED;
    }
    if (control_block.upgrade_flag == UPGRADE_NSET)
        return 0;

    debug_upgrade_printf("upgrade_flag is set, will_select is %s\n\r",
                         (control_block.will_select_flag == IMAGE_A) ? "IMAGE_A" : "IMAGE_B");

    if (control_block.will_select_flag != current_running_image())
    {
        printf("current running image is not will_select_flag\n\r");
        return FAILED;
    }

    if (control_block.will_select_flag == IMAGE_A)
    {
        control_block.imageA_loader_count = 0;
        control_block.imageA_valid_flag = IMAGE_VALID;
    }
    else
    {
        control_block.imageB_loader_count = 0;
        control_block.imageB_valid_flag = IMAGE_VALID;
    }
    control_block.select_flag = control_block.will_select_flag;
    control_block.upgrade_flag = UPGRADE_NSET;
    control_block.crc_32bit = control_block_crc_compute(&control_block);
    if (control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BASE, (ctrl_info_t *) & control_block))
    {
        debug_upgrade_printf("copy CONTROL_BLOCK_BASE failed\n\r");
        return FAILED;
    }
    return control_block_backup_save();
}

uint32_t switchover_image_check()
{
    ctrl_info_t *control_block = (ctrl_info_t *) CONTROL_BLOCK_BASE;
    if (control_block_verification(control_block))
    {
        printf("CONTROL_BLOCK_BASE crc error, should not happen!\n\r");
        return FAILED;
    }
    if ((control_block->imageA_valid_flag != IMAGE_VALID) || (control_block->imageB_valid_flag != IMAGE_VALID))
    {
        debug_upgrade_printf("another image is not verified!\n\r");
        return FAILED;
    }

    if (image_crc_check(IMAGE_A))
    {
        printf("IMAGE_A check failed!\n\r");
        return FAILED;
    }

    if (image_crc_check(IMAGE_B))
    {
        printf("IMAGE_B check failed!\n\r");
        return FAILED;
    }

    return 0;
}

uint32_t switchover_image_operation()
{
    ctrl_info_t control_block;

    if (switchover_image_check())
        return FAILED;

    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BASE, sizeof(control_block));

    if (control_block.select_flag == IMAGE_A)
    {
        control_block.select_flag = IMAGE_B;
        debug_upgrade_printf("do switchover from IMAGE_A to IMAGE_B\n\r");
    }
    else
    {
        control_block.select_flag = IMAGE_A;
        debug_upgrade_printf("do switchover from IMAGE_B to IMAGE_A\n\r");
    }
    control_block.will_select_flag = control_block.select_flag;
    control_block.crc_32bit = control_block_crc_compute(&control_block);
    if (control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BASE, (ctrl_info_t *) & control_block))
    {
        debug_upgrade_printf("copy CONTROL_BLOCK_BASE failed\n\r");
        return FAILED;
    }
    return control_block_backup_save();
}

uint32_t whether_can_upgrade_image()
{
    ctrl_info_t *control_block_p;

    control_block_p = (ctrl_info_t *) CONTROL_BLOCK_BASE;
    if (control_block_p->select_flag == current_running_image())
        return 1;
    else
        return 0;
}

uint32_t current_select_image()
{
    ctrl_info_t *control_block_p;

    control_block_p = (ctrl_info_t *) CONTROL_BLOCK_BASE;

    if (control_block_p->upgrade_flag == UPGRADE_SET)
    {
        return control_block_p->will_select_flag;
    }
    else
    {
        return control_block_p->select_flag;
    }
}

void image_header_dump(image_header_t * image_header_p)
{
    if (image_header_check(image_header_p))
    {
        printf("image header crc error\n\r");
        return;
    }
    printf("magic: %s\n\r", image_header_p->magic);
    printf("image_version: %s\n\r", image_header_p->image_version);
    printf("build_time: %s\n\r", image_header_p->build_time);
    printf("oem_id: %s\n\r", image_header_p->oem_id);
    printf("product_id: %s\n\r", image_header_p->product_id);
    printf("image_header_length: 0x%x\n\r", image_header_p->image_header_length);
    printf("image_length: 0x%x\n\r", image_header_p->image_length);
    printf("image_checksum: 0x%x\n\r", image_header_p->image_checksum);
    printf("header_checksum: 0x%x\n\r", image_header_p->header_checksum);
}

void control_info_dump(ctrl_info_t * control_block_p)
{
    if (control_block_verification(control_block_p))
    {
        printf("control block crc error\n\r");
        return;
    }
    printf("upgrade_flag: %s\n\r", (control_block_p->upgrade_flag == UPGRADE_SET) ? "UPGRADE_SET" : "UPGRADE_NSET");
    printf("select_flag: %s\n\r", (control_block_p->select_flag == IMAGE_A) ? "IMAGE_A" : "IMAGE_B");
    printf("will_select_flag: %s\n\r", (control_block_p->will_select_flag == IMAGE_A) ? "IMAGE_A" : "IMAGE_B");
    printf("imageA_loader_count: %d\n\r", control_block_p->imageA_loader_count);
    printf("imageA_valid_flag: %s\n\r",
           (control_block_p->imageA_valid_flag == IMAGE_VALID) ? "IMAGE_VALID" : "IMAGE_UNVALID");
    printf("imageB_loader_count: %d\n\r", control_block_p->imageB_loader_count);
    printf("imageB_valid_flag: %s\n\r",
           (control_block_p->imageB_valid_flag == IMAGE_VALID) ? "IMAGE_VALID" : "IMAGE_UNVALID");
}

uint32_t upgrade_info_dump()
{
    printf("control block info:\n\r");
    control_info_dump((ctrl_info_t *) CONTROL_BLOCK_BASE);
    printf("\n\r");

    printf("control block backup1 info:\n\r");
    control_info_dump((ctrl_info_t *) CONTROL_BLOCK_BACKUP1_ADDR);
    printf("\n\r");

    printf("control block backup2 info:\n\r");
    control_info_dump((ctrl_info_t *) CONTROL_BLOCK_BACKUP2_ADDR);
    printf("\n\r");

    printf("imageA head info:\n\r");
    image_header_dump((image_header_t *) IMAGE_A_BASE);
    printf("\n\r");

    printf("imageB head info:\n\r");
    image_header_dump((image_header_t *) IMAGE_B_BASE);
    printf("\n\r");
    return 0;
}


uint32_t debug_set_ctrl_block(char *image_name)
{
    ctrl_info_t control_block;
    uint32_t image;
    printf("image_name:%s\n\r", image_name);
    image = (strcmp(image_name, "imageA") == 0) ? IMAGE_A : IMAGE_B;
    printf("we select %s\n\r", (image == IMAGE_A) ? "IMAGE_A" : "IMAGE_B");

    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BASE, sizeof(control_block));
    control_block.upgrade_flag = UPGRADE_NSET;
    control_block.will_select_flag = image;
    control_block.select_flag = image;

    if (image == IMAGE_A)
    {
        control_block.imageA_loader_count = 0;
        control_block.imageA_valid_flag = IMAGE_VALID;
    }
    else
    {
        control_block.imageB_loader_count = 0;
        control_block.imageB_valid_flag = IMAGE_VALID;
    }
    control_block.crc_32bit = control_block_crc_compute(&control_block);
    debug_upgrade_printf("control_block@0x%x crc_32bit:0x%x@0x%x\n\r", (uint32_t) & control_block,
                         control_block.crc_32bit, (uint32_t) & control_block.crc_32bit);
    control_info_dump(&control_block);
    return control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BASE, (ctrl_info_t *) & control_block);
}

uint32_t debug_crash_ctrl_block()
{
    ctrl_info_t control_block;
    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BASE, sizeof(control_block));
    control_block.select_flag = 0;
    return control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BASE, (ctrl_info_t *) & control_block);
}

uint32_t debug_crash_backup1_block()
{
    ctrl_info_t control_block;
    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BACKUP1_ADDR, sizeof(control_block));
    control_block.select_flag = 0;
    return control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BACKUP1_ADDR, (ctrl_info_t *) & control_block);
}

uint32_t debug_crash_backup2_block()
{
    ctrl_info_t control_block;
    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BACKUP2_ADDR, sizeof(control_block));
    control_block.select_flag = 0;
    return control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BACKUP2_ADDR, (ctrl_info_t *) & control_block);
}

uint32_t debug_upgrade_image(uint32_t source, uint32_t dest)
{
    ctrl_info_t control_block;
    image_header_t *image_header_p;
    uint8_t *from, *src;
    uint32_t addr, image, dst = 0;
    uint32_t i, len, total = 0, len_this_lp, bytes_this_page;
    uint32_t need_buf = 0;
    uint8_t date_buf[MUSIC_SPI_PAGE_SIZE];

    if ((source >= SPI_FLASH_BASE) || (source <= SPI_FLASH_BASE + SPI_FLASH_SIZE))
        need_buf = 1;

    image_header_p = (image_header_t *) source;
    if (image_header_check(image_header_p))
    {
        printf("image header analyze failed @0x%x\n\r", source);
        return FAILED;
    }
    len = image_header_p->image_length + IMAGE_HEADER_SIZE;
    if (len % 4)
    {
        printf("image_length:0x%x should be devided by 4\n\r", len);
        return FAILED;
    }

    addr = dest - SPI_FLASH_BASE;
    image = ((dest == IMAGE_A_BASE) ? IMAGE_A : IMAGE_B);


    while (total < len)
    {
        src = (uint8_t *) (source + total);
        dst = addr + total;
        bytes_this_page = MUSIC_SPI_PAGE_SIZE - (addr % MUSIC_SPI_PAGE_SIZE);
        len_this_lp = ((len - total) > bytes_this_page) ? bytes_this_page : (len - total);
#if 0
        if (total % (MUSIC_SPI_PAGE_SIZE * 0x80) == 0)
            printf("src:%x dst:%x len_this_lp:%x total:%x len:%x\n", src, dst | 0xb9000000, len_this_lp, total, len);
#endif
        from = src;
        if ((dst % MUSIC_SPI_SECTOR_SIZE) == 0)
        {
            if ((dst % (MUSIC_SPI_SECTOR_SIZE * 0x20)) == 0)
                printf("erase sector @0x%x\n\r", SPI_FLASH_BASE + dst);
            spiflash_sector_erase(dst);
        }

        if (need_buf)
        {
            for (i = 0; i < len_this_lp; i++)
            {
                date_buf[i] = *from++;
            }
            spiflash_write_page(dst, date_buf, len_this_lp);
        }
        else
            spiflash_write_page(dst, (uint8_t *) src, len_this_lp);
        total += len_this_lp;
    }
    printf("complete image burn\n\r");

    if (image_crc_check(image))
    {
        printf("image checksum is not correct\n\r");
        return FAILED;
    }


    memcpy((void *)&control_block, (void *)CONTROL_BLOCK_BASE, sizeof(control_block));
    control_block.upgrade_flag = UPGRADE_SET;
    control_block.will_select_flag = image;

    if (image == IMAGE_A)
    {
        control_block.imageA_loader_count = 0;
        control_block.imageA_valid_flag = IMAGE_UNVALID;
    }
    else
    {
        control_block.imageB_loader_count = 0;
        control_block.imageB_valid_flag = IMAGE_UNVALID;
    }
    control_block.crc_32bit = control_block_crc_compute(&control_block);
    debug_upgrade_printf("control_block@0x%x crc_32bit:0x%x@0x%x\n\r", (uint32_t) & control_block,
                         control_block.crc_32bit, (uint32_t) & control_block.crc_32bit);
    control_info_dump(&control_block);
    return control_block_copy((ctrl_info_t *) CONTROL_BLOCK_BASE, (ctrl_info_t *) & control_block);
}

void show_image_info()
{
    uint32_t current_image = current_running_image();
    uint32_t image_heaader_addr = (current_image == IMAGE_A) ? IMAGE_A_BASE : IMAGE_B_BASE;
    char image_version[IMAGE_VERSION_LEN];
    char serial_num[SERIAL_NUM_LEN];
    char mac[MAC_ADDRESS_LEN];

    /*
     * maybe this image is run in memory, not in spi flash
     */
    if (current_image == FAILED)
        return;
    image_header_t *image_header_p = (image_header_t *) image_heaader_addr;
    image_get_version(image_version);
    product_get_serial_number(serial_num);
    product_get_mac(mac);
    printf("%s running ", (current_image == IMAGE_A) ? "IMAGE_A" : "IMAGE_B");
    printf("%s build time:%s\n\r", image_header_p->magic, image_header_p->build_time);
    printf("image_version:%s\n", image_version);
    printf("product_serial_number:%s\n", serial_num);
    printf("product_mac_address:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x:%2.2x\n",
           (uint8_t) mac[0], (uint8_t) mac[1], (uint8_t) mac[2], (uint8_t) mac[3], (uint8_t) mac[4], (uint8_t) mac[5]);
}
int image_get_version(char *version)
{
    uint32_t current_image = current_running_image();
    uint32_t image_heaader_addr = (current_image == IMAGE_A) ? IMAGE_A_BASE : IMAGE_B_BASE;
    if (version == NULL)
        return -1;
    image_header_t *image_header_p = (image_header_t *) image_heaader_addr;
    strncpy(version, image_header_p->image_version, IMAGE_VERSION_LEN);
    return 0;
}

int image_get_version_2(uint32_t image, char *version)
{
    uint32_t image_heaader_addr = (image == IMAGE_A) ? IMAGE_A_BASE : IMAGE_B_BASE;
    if (version == NULL)
        return -1;
    image_header_t *image_header_p = (image_header_t *) image_heaader_addr;
    strncpy(version, image_header_p->image_version, IMAGE_VERSION_LEN);
    return 0;
}

int product_get_serial_number(char *serial)
{
    board_info_t *board_info_p = (board_info_t *) BOARD_INFO_BASE;
    if (serial == NULL)
        return -1;
    strncpy(serial, board_info_p->serial_num, SERIAL_NUM_LEN);
    return 0;
}

int product_get_mac(char *mac)
{
    int i;
    board_info_t *board_info_p = (board_info_t *) BOARD_INFO_BASE;
    if (mac == NULL)
        return -1;
    for (i = 0; i < MAC_ADDRESS_LEN; i++)
        mac[i] = board_info_p->mac_address[i];
    return 0;
}

int product_get_product_id(char *id)
{
    board_info_t *board_info_p = (board_info_t *) BOARD_INFO_BASE;
    if (id == NULL)
        return -1;
    strncpy(id, board_info_p->serial_num, PRODUCT_LEN);
    return 0;
}
#endif

#if 0
uint32_t elite_running_seconds()
{
    return running_seconds;
}
#endif
