/* 
 * Copyright (c) 2012 Qualcomm Atheros, Inc.. 
 * All Rights Reserved. 
 * Qualcomm Atheros Confidential and Proprietary. 
 */

#ifdef IN_LOADER
#include "music_soc.h"
#endif
#include "watchdog.h"
#include "music_util.h"

#define IN_LOADER
#ifndef IN_LOADER
#include "contiki.h"
#endif

#ifdef IN_LOADER
#define KUSEG			0x00000000
#define KSEG0			0x80000000
#define KSEG1			0xa0000000
#define KSEG2			0xc0000000
#define KSEG3			0xe0000000

#define KSEG0ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG0))
#define KSEG1ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG1))
#define KSEG2ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG2))
#define KSEG3ADDR(a)		((__typeof__(a))(((unsigned long)(a) & 0x1fffffff) | KSEG3))

#define music_reg_rd(_phys)    (*(volatile unsigned int *)KSEG1ADDR(_phys))
#define music_reg_wr_nf(_phys, _val) \
                    ((*(volatile unsigned int *)KSEG1ADDR(_phys)) = (_val))

#define music_reg_wr(_phys, _val) do {     \
                    music_reg_wr_nf(_phys, _val);  \
                    music_reg_rd(_phys);       \
}while(0);

#endif

#define MUSIC_WD_REGISTER_UNLOCK() do { \
        music_reg_wr(MUSIC_WATCHDOG_LOCK_REG, MUSIC_WD_UNLOCK_NUM); \
        }while(0)

#define MUSIC_WD_REGISTER_LOCK() do { \
        music_reg_wr(MUSIC_WATCHDOG_LOCK_REG, MUSIC_WD_LOCK_NUM); \
        }while(0)

typedef unsigned int uint32_t;

uint32_t running_loop = 0, last_running_clock = 0, running_seconds = 0;

extern int soc_bus_freq_get(void);

int music_set_wd_timer_action(uint32_t action)
{
    uint32_t reg_val = 0;
    if (action & (~MUSIC_WD_ACT_MASK))
    {
        return -1;
    }

    MUSIC_WD_REGISTER_UNLOCK();

    reg_val = music_reg_rd(MUSIC_WATCHDOG_CTRL_REG);

    reg_val &= (~MUSIC_WD_ACT_MASK);
    reg_val |= action;

    music_reg_wr(MUSIC_WATCHDOG_CTRL_REG, reg_val);

    MUSIC_WD_REGISTER_LOCK();
    return 0;
}

int music_set_wd_timer(uint32_t time)
{
    MUSIC_WD_REGISTER_UNLOCK();
    music_reg_wr(MUSIC_WATCHDOG_LOAD_REG, time);
    MUSIC_WD_REGISTER_LOCK();

    return 0;
}

int music_set_clr_en(void)
{
    uint32_t reg_val = 0;

    MUSIC_WD_REGISTER_UNLOCK();

    reg_val = music_reg_rd(MUSIC_WATCHDOG_CTRL_REG);
    reg_val |= BIT2;

    music_reg_wr(MUSIC_WATCHDOG_CTRL_REG, reg_val);
    MUSIC_WD_REGISTER_LOCK();

    return 0;
}

int music_set_clr_dis(void)
{
    uint32_t reg_val = 0;

    MUSIC_WD_REGISTER_UNLOCK();

    reg_val = music_reg_rd(MUSIC_WATCHDOG_CTRL_REG);
    reg_val &= (~BIT2);

    music_reg_wr(MUSIC_WATCHDOG_CTRL_REG, reg_val);

    MUSIC_WD_REGISTER_LOCK();
    return 0;
}

int music_clear_wd_timer(void)
{
    MUSIC_WD_REGISTER_UNLOCK();
    music_reg_wr(MUSIC_WATCHDOG_CLR_REG, 1);
    MUSIC_WD_REGISTER_LOCK();
    return 0;
}


void watchdog_init(void)
{
    uint32_t watchdog_time_count;
    uint32_t frequency_devision = 1;

    /*
     * 
     */
    frequency_devision = (music_reg_rd(MUSIC_WATCHDOG_CTRL_REG) & MUSIC_WD_STEP_COUNTER_MASK)
        >> MUSIC_WD_STEP_COUNTER_SHIFT;
    frequency_devision++;
#ifdef IN_LOADER
    //in loader we set watchdog timeout count to 5 seconds
    watchdog_time_count = soc_bus_freq_get() / frequency_devision;
    watchdog_time_count = watchdog_time_count * 5 / 2;
#else
    watchdog_time_count = soc_bus_freq_get() / frequency_devision;
    watchdog_time_count = watchdog_time_count * MUSIC_DEFAULT_WD_TIMER_SECONDS / 2;
#endif
    music_set_wd_timer_action(MUSIC_WD_ACT_NONE);
    music_set_wd_timer(watchdog_time_count);
    music_set_clr_en();
}

void watchdog_start(void)
{
    music_clear_wd_timer();
    music_set_wd_timer_action(MUSIC_WD_ACT_RESET);
}

void watchdog_periodic(void)
{
    music_clear_wd_timer();
}

void watchdog_stop(void)
{
    music_set_wd_timer_action(MUSIC_WD_ACT_NONE);
}

#ifndef IN_LOADER
void watchdog_dump(void)
{
    printf("MUSIC_WATCHDOG_LOAD_REG: 0x%8.8x\n", music_reg_rd(MUSIC_WATCHDOG_LOAD_REG));
    printf("MUSIC_WATCHDOG_VALUE_REG: 0x%8.8x\n", music_reg_rd(MUSIC_WATCHDOG_VALUE_REG));
    printf("MUSIC_WATCHDOG_CTRL_REG: 0x%8.8x\n", music_reg_rd(MUSIC_WATCHDOG_CTRL_REG));
    printf("MUSIC_WATCHDOG_CLR_REG: 0x%8.8x\n", music_reg_rd(MUSIC_WATCHDOG_CLR_REG));
    printf("MUSIC_WATCHDOG_RIS_REG: 0x%8.8x\n", music_reg_rd(MUSIC_WATCHDOG_RIS_REG));
    printf("MUSIC_WATCHDOG_MIS_REG: 0x%8.8x\n", music_reg_rd(MUSIC_WATCHDOG_MIS_REG));
    printf("MUSIC_WATCHDOG_LOCK_REG: 0x%8.8x\n", music_reg_rd(MUSIC_WATCHDOG_LOCK_REG));
}

void update_system_time(uint32_t current_clock)
{
    if (current_clock < last_running_clock)
        running_loop++;
    last_running_clock = current_clock;

    running_seconds = current_clock / CLOCK_CONF_SECOND;
    running_seconds += (0xffffffff / CLOCK_CONF_SECOND) * running_loop;
}

static process_event_t event_data_ready;
PROCESS(watchdog_process, "watchdog");

PROCESS_THREAD(watchdog_process, ev, data)
{
    PROCESS_BEGIN();
    static struct etimer timer;

    etimer_set(&timer, CLOCK_CONF_SECOND * 5);
    printf("+++++++++++++++++watchdog process start++++++++++++++++\n");

    event_data_ready = process_alloc_event();

    while (1)
    {
        PROCESS_WAIT_EVENT_UNTIL(ev == PROCESS_EVENT_TIMER);
        update_system_time(clock_time());
        watchdog_periodic();
        etimer_reset(&timer);
    }

    PROCESS_END();
}
#endif
